package Ockham::Alert::Description;

use strict;
use warnings;
use base qw( Ockham::Alert::DB );
use Ockham::Alert::Utils qw( normalize );
use Ockham::Alert::Config qw( DB_DRIVER );

=head1 NAME

Ockham::Alert::Description - Represent a single description of a resource

=head1 SYNOPSIS

    my $resource = Ockham::Alert::Resource->retrieve( $id );
    foreach my $description ( $resource->descriptions() ) {
        print $description->property(), ': ', $description->value(), "\n";
    }

=head1 DESCRIPTION

Ockham::Alert::Description represents a single description attached to a 
Ockham::Alert::Resource object. For example a title of Huckleberry Finn
would be one description, and author of Samuel Clemens. It is possible
for descriptions to repeat, so you could have an additional description to
represent author of Mark Twain.

Ockham::Alert::Description uses Class::DBI to provide access to the DESCRIPTION
table in the database. Below are the methods you are likely to call in addition
to all the ones made available to you as part of the Class::DBI framework.

=head1 METHODS

=head1 

=head2 property()

The property of the description. Examples include 'creator', 'title', 'date'
etc. These will typically be one of the 15 Dublin Core elements.

=head2 qualifier()

A qualifier (if any) associated with the property. An example of a qualifier
could be 'created' which qualifies a description with property of 'date'.
Typically these will be qualifiers recommended by Dublin Core, but there is 
no restriction in the code that prevents you from putting whatever you want in
here.

=head2 value()

The actual value of the description. Examples could include 'Mark Twain' when
the property is set to 'creator'.

=head2 normal_value()

A normalized version of the value potentially used in searching.

=head2 resource()

Returns the Ockham::Alert::Resource object that the description refers to. 

=cut

__PACKAGE__->table( 'description' );
__PACKAGE__->columns( Primary => qw( id ) );
__PACKAGE__->columns( Essential => qw( 
    resource 
    property 
    qualifier 
    value 
    normal_value 
) );

__PACKAGE__->sequence( 'description_id_seq' ) if DB_DRIVER eq 'Pg';
__PACKAGE__->add_trigger( before_create => \&_normalize );
__PACKAGE__->add_trigger( before_update => \&_normalize );

sub normalize_column_values {
    my ( $class, $cols ) = @_;
    if ( exists( $cols->{ value } ) and length( $cols->{ value } ) > 2500 ) { 
        $cols->{ value } = substr( $cols->{ value }, 0, 2500 );
    }
}

sub _normalize {
    my $self = shift;
    $self->normal_value( normalize( $self->value() ) );
}

1;
