package Ockham::Alert::Provider;

use strict;
use warnings;
use base qw( Ockham::Alert::DB );
use Net::OAI::Harvester;
use Ockham::Alert::Utils qw( tag );
use Ockham::Alert::Config qw( DB_DRIVER );
use DateTime;

__PACKAGE__->table( 'provider' );
__PACKAGE__->columns( Primary => qw( id ) );
__PACKAGE__->columns( All => qw( created name type url ) );
__PACKAGE__->has_a( created => 'DateTime', 
    inflate => sub { DateTime->from_epoch( epoch => shift ); },
    deflate => sub { shift->epoch(); } );
__PACKAGE__->sequence( 'provider_id_seq' ) if DB_DRIVER eq 'Pg';
__PACKAGE__->add_trigger( before_create => \&_getInfo );

=head2 harvester() 

Factory method that will return a Net::OAI::Harvester object for the 
provider.

=cut

sub harvester { 
    my $self = shift;
    return( Net::OAI::Harvester->new( baseUrl => $self->url() ) );
}

=head2 asXML()

A method that returns a particular provider serialized as XML.

=cut

sub asXML {
    my $self = shift;
    my $xml = "<provider>\n";
    $xml .= 
        tag( 'title', '', $self->name() ) .
        tag( 'format', '', $self->type() ) .
        tag( 'identifier', 'provider', $self->url() );
    $xml .= "</provider>\n";
    return( $xml );
}

## when a provider is being created _getInfo() will poll OAI-PMH
## repositories for a bit of information.

sub _getInfo { 
    my $self = shift;
    $self->created( DateTime->now() );
    return if $self->type() ne 'oai';

    ## create oai harvester
    my $harvester = Net::OAI::Harvester->new( baseUrl => $self->url() );

    ## fetch provider object for repository or create it if we 
    ## don't already know about it 

    my $identity = $harvester->identify();
    my $name = $identity->repositoryName() || "Unknown Repository Name";
    $self->name( $name );
}

1;
