package Ockham::Alert::Resource;

use strict;
use warnings;
use base qw( Ockham::Alert::DB );
use Carp qw( croak );
use DateTime;
use Ockham::Alert::Description;
use Ockham::Alert::Utils qw( tag );
use Ockham::Alert::Indexer::Plucene;
use Ockham::Alert::Config qw( DB_DRIVER );

=head1 NAME

Ockham::Alert::Resource - Object for representing a Resource

=head1 SYNOPSIS

    use Ockham::Alert::Resource;
    my $resource = Ockham::Alert::Resource->retrieve( $id );
    print $resource->oai_id();

=head1 DESCRIPTION

Ockham::Alert::Resource represents information resources that are in the 
alerting database. They correspond to rows in the RESOURCE table, as well
as corresponding rows in the DESCRIPTOION table. For each resource there can be
many descriptions.

=head1 METHODS

=head2 id()

The database identifier for the resource object.

=head2 oai_id()

An OAI-PMH identifier, if present.

=head2 created()

The date the resource was created. This is the date the original metadata
record was created, not the date the record was added to the alerting system. 

=head2 harvested()

The date the resource was harvested, or added to the alerting system.

=head2 provider()

Returns an Ockham::Alert::Provider object for the provider that supplied this
record.

=head2 descriptions()

Returns a list of Ockham::Alert::Description objects for the resource.

=cut

__PACKAGE__->table( 'resource' );
__PACKAGE__->columns( Essential => qw( id oai_id created harvested provider ) );
__PACKAGE__->sequence( 'resource_id_seq' ) if DB_DRIVER eq 'Pg';
__PACKAGE__->has_a( provider => 'Ockham::Alert::Provider' );
__PACKAGE__->has_a( created => 'DateTime',
    inflate => sub { DateTime->from_epoch( epoch => shift ); },
    deflate => sub { shift->epoch(); } );
__PACKAGE__->has_a( harvested => 'DateTime',
    inflate => sub { DateTime->from_epoch( epoch => shift ); },
    deflate => sub { shift->epoch(); } );
__PACKAGE__->has_many( descriptions => 'Ockham::Alert::Description' );
__PACKAGE__->add_trigger(
    'before_create' => sub { shift->harvested( DateTime->now() ); } );
__PACKAGE__->add_trigger( 
    'after_create' => \&Ockham::Alert::Indexer::Plucene::addToIndex );

my @DC_FIELDS = qw(
    title creator subject description publisher contribtor
    date type format identifier source language relation
    coverage rights
);

=head2 field()

Shortcut to retrieve metadata fields for a resource by the name of the field.
In a list context multiple fields are returned if appropriate (for example 
when there are mutltiple subjects). In a scalar context only the first is 
returned.

    my $title = $resource->field( 'title' );
    my @subjects = $resource->field( 'subjects' );

=cut

sub field {
    my ( $self, $property ) = @_;
    my @descriptions = $self->descriptions( property => $property );
    if ( wantarray() ) { return( map { $_->value() } @descriptions ) }
    elsif ( @descriptions > 0 ) { return $descriptions[0]->value(); }
    return;
}

=head2 createFromOAIRecord()

=cut

sub createFromOAIRecord {
    my ( $class, %opts ) = @_;

    ## check some stuff
    croak( "must call as class method" ) 
        if $class ne 'Ockham::Alert::Resource';
    croak( "must pass record parameter" ) 
        if ! exists( $opts{ record } );
    croak( "record parameter must be a Net::OAI::Record object" )
        if ref( $opts{ record } ) !~ /Net::OAI::GetRecord|Net::OAI::Record/;
    croak ( "must pass provider parameter" )
        if ! exists( $opts{ provider } );
    croak( "provider paramter must be a Ockham::Alert::Provider object" )
        if ref( $opts{ provider } ) ne 'Ockham::Alert::Provider';

    my $header = $opts{ record }->header();
    my $metadata = $opts{ record }->metadata();
    my $provider = $opts{ provider };

    ## figure out the day the record was added
    my ( $year, $month, $day ) = $header->datestamp() 
        =~ /^(\d{4})-(\d{2})-(\d{2})/;
        my $date = DateTime->new( 
            year    => $year, 
            month   => $month, 
            day     => $day 
        );
        
    ## create a resource object
    my $resource = Ockham::Alert::Resource->create( {
        provider        => $provider,
        oai_id          => $header->identifier(),
        created         => $date, 
    } );

    ## attach dublin core fields to the resource
    foreach my $property ( @DC_FIELDS ) { 
        foreach my $value ( $metadata->$property() ) {
            $resource->add_to_descriptions( {
                property        => $property,
                value           => $value,
            } );
        }
    }

    ## return the new resource object
    return( $resource );
}

=head2 createFromXML()

A factory method for creating a Ockham::Alert::Resource object from a chunk 
of XML.

=cut

sub fromXML {
}

=head2 asXML()

Dumps a OAI::Alert::Resource object as XML.

=cut

sub asXML {
    my $self = shift;
    my $xml = "<resource>\n";
    $xml .= 
        tag( 'identifier', 'alertResource', $self->id() ) .
        tag( 'date', 'created', $self->created->datetime() ) .
        tag( 'date', 'harvested', $self->harvested->datetime() ) .
        tag( 'publisher', 'provider', $self->provider->name() ) .
        tag( 'identifier', 'oai', $self->oai_id() );
    foreach my $description ( $self->descriptions() ) { 
        $xml .= tag( $description->property(), '' , $description->value() );
    }
    $xml .= "</resource>\n";
    return( $xml );
}

1;
