#!/usr/bin/perl

# client.cgi - a specialized SRU client returning HTML, email, and RSS feeds from queries

# Eric Lease Morgan <emorgan@nd.edu>
# Fall, 2005

# pod

=head1 NAME

client.cgi


=head1 DESCRIPTION

This is a specialized SRU (Search/Retrieve via URL) client that returns results as HTML, RSS, and email messages.


=head1 INSTALLATION

=over

=item 1.

Edit the value of SRUROOT to point to your Ockham Alert SRU server.

=item 2.

Edit the value of SMTP to point to your... SMTP server.

=item 3.

Edit the various etc/*.txt files. These are template files complete with tokens. The tokens get replaced with values as this script gets executed. Begin by editing etc/template.txt.

=item 4.

Edit the various etc/*.xsl files. These are XSL stylesheets transforming the SRU responses into HTML, RSS, and email messages.

=back


=head1 AUTHOR

Eric Lease Morgan <emorgan@nd.edu> & Team Ockham (Fall, 2005)

=cut



# require/use
use CGI;
use LWP::UserAgent;
use MIME::Lite;
use strict;
use XML::LibXML;
use XML::LibXSLT;

# define constants
use constant SRUROOT   => 'http://alert.ockham.org/sru-server.cgi?operation=searchRetrieve&version=1.1';
use constant SRU2HTML  => 'etc/sru2html.xsl';
use constant SRU2EMAIL => 'etc/sru2email.xsl';
use constant SRU2RSS   => 'etc/sru2rss.xsl';
use constant TEMPLATE  => 'etc/template.txt';
use constant HOME      => 'etc/home.txt';
use constant EMAIL     => 'etc/email.txt';
use constant SMTP      => 'smtp.library.oregonstate.edu';

# initialize
my $cgi = CGI->new;
my $xml = '';

# a hack to see who is using this application
#open (MAIL, "| mail -s '[alert] $ENV{REMOTE_HOST}' emorgan\@nd.edu");
#print MAIL qq($ENV{REMOTE_ADDR} ($ENV{REMOTE_HOST})\n$ENV{HTTP_USER_AGENT}\n\n$ENV{QUERY_STRING});
#close MAIL;

# get the command
my $cmd = $cgi->param('cmd');

# branch accordingly; apparently i should be using CGI::Application here
if (! $cmd) {

	# display the home page
	$xml = &slurp(TEMPLATE);
	$xml =~ s/##CONTENT##/&slurp(HOME)/e;
	$xml =~ s/##QUERY##//e;
		
}

elsif ($cmd eq 'search') {
	
	# get the query
	my $query = $cgi->param('query');
	
	# do simple munging of the query; try to force it into CQL
	if ($query =~ /\s/) {
	
		if (($query =~ / and /) | ($query =~ / or /) | ($query =~ / not /)) { }		
		elsif ($query =~ /=/) { }
		elsif ($query !~ /"/) { $query = '"' . $query . '"' }
		
	}
	
	# create a user agent, create a request, send it, and get a response
	my $ua = LWP::UserAgent->new(agent => 'SRU-Client/0.1 ');
  	my $request = HTTP::Request->new(GET => SRUROOT . '&query=' . $query);
	
	my $response = $ua->request($request);
	
	# check response
	if ($response->is_success) {
			
		# branch accordingly
		if ($cgi->param('output') eq 'SRU') {
		
			# give them the raw data
			$xml = $response->content
		
		}
		
		elsif ($cgi->param('output') eq 'HTML') {
		
			# get the template and fill it with the transformed response, html
			$xml = &slurp(TEMPLATE);
			$xml =~ s/##CONTENT##/&transform(xslt => SRU2HTML, xml => $response->content)/e;
			$xml =~ s/##QUERY##/$cgi->param('query')/ge;
			
		}
		
		elsif ($cgi->param('output') eq 'RSS') {
		
			# get the template and fill it with the transformed response, rss
			$xml = &transform(xslt => SRU2RSS, xml => $response->content)
			
		}
			
		elsif ($cgi->param('output') eq 'Email') {
		
			# build a message and send it
			my $msg = MIME::Lite->new(From    => 'Ockham Alerting Service <emorgan@nd.edu>', 
			                          To      => $cgi->param('address'),
			                          Subject => 'Ockham Alerting Service search results',
			                          Type    => 'text/html',
			                          Data    => &transform(xslt => SRU2EMAIL, xml => $response->content)
			                         );
            $msg->send("smtp", SMTP);
            
			# display the home page
			$xml = &slurp(TEMPLATE);
			$xml =~ s/##CONTENT##/&slurp(EMAIL)/e;
			$xml =~ s/##QUERY##/$cgi->param('query')/ge;
			$xml =~ s/##ADDRESS##/$cgi->param('address')/e;

		}
		
		else { }
		
	}
	
	else { $xml = $response->status_line, "\n" }
	
}

else {

	# error
	$xml = $cgi->start_html(-title => 'SRU client front-end');
	$xml .= $cgi->h1('SRU client front-end');
	$xml .= $cgi->p("Unknown value for cmd ($cmd). Call Eric.");
	$xml .= $cgi->end_html;

}


# done
print $cgi->header('text/xml');
print $xml;


sub transform {

	my %opts = @_;

	my $parser     = XML::LibXML->new;
	my $xslt       = XML::LibXSLT->new;
	my $source     = $parser->parse_string($opts{xml}) or croak $!;
	my $style      = $parser->parse_file($opts{xslt}) or croak $!;
	my $stylesheet = $xslt->parse_stylesheet($style) or croak $!;
	my $results    = $stylesheet->transform($source) or croak $!;
	
	return $stylesheet->output_string($results);


}



sub slurp {

	my $f = @_[0];
	my $r;
	
	open (F, "< $f");
	while (<F>) { $r .= $_ }
	close F;
	
	return $r;

}
