#!/usr/bin/perl

# sru-server.cgi - the beginings of an SRU server
# Eric Lease Morgan <emorgan@nd.edu>

# December 23, 2004 - tweaking zeerex record; got diagnostics to work
# December 17, 2004 - moving to ockham.org
# December 15, 2004 - playing with no stylesheets and CQL toSwish
# September 7, 2004 - hacking w/ a flu, sort of
# September 5, 2004 - first investigations

# pod

=head1 NAME

server.cgi


=head1 DESCRIPTION

This is a rudimentary SRU (Search/Retrieve via URL) server written in Perl. It is expected to provide access to an index created by swish-e index (www.swish-e.org) containing content harvested from OAI repositories.


=head1 INSTALLATION

There really isn't any configuration necessary, but strictly speaking you need to edit the contents of the record subroutine to describe characteristics of your SRU implementation.


=head1 AUTHOR

Eric Lease Morgan <emorgan@nd.edu> & Team Ockham (Fall, 2004)

=cut


# require the necessary modules
use CGI qw(-oldstyle_urls);
use CGI::Carp qw(fatalsToBrowser);
use CQL::Parser;
use Ockham::Alert::Config qw( SWISH_INDEX );
use Ockham::Alert::Resource;
use SRU::Request;
use SRU::Response;
use strict;
use SWISH::API;
use warnings;

# initlize the necessary objects
my $cgi      = CGI->new();
my $request  = SRU::Request->newFromCGI($cgi)          || &error;
my $response = SRU::Response->newFromRequest($request) || &error;

# check for type of response; explain
if ($response->type() eq 'explain') {
	
	# fill up the response's record 
	$response->record(SRU::Response::Record->new(recordSchema => 'http://explain.z3950.org/dtd/2.0/',
	                                               recordData => &record));

}

# scan
elsif ($response->type() eq 'scan') {
	
	$response->addDiagnostic(SRU::Response::Diagnostic->newFromCode(4, 'Scan operation is not supported at this stage.'));
	$response->asXML();
}

# search
else {
	
	# parse the query and check it
	my $parser = CQL::Parser->new;
	eval { $parser->parse($request->query) };
    if ($@) {
    
    	$response->addDiagnostic(SRU::Response::Diagnostic->newFromCode(10,$@));
        
        print $cgi->header('text/xml');
		print $response->asXML();
		exit;

     }
     
    my $node = $parser->parse($request->query);
	
	# search
	my @results = &search($node->toSwish());
	
	# process each result
	for (my $i = 0; $i <= $#results; $i++) {

		# check for maximum records
		if (defined($request->maximumRecords)) { last if ($i >= $request->maximumRecords) }
		
		# create a records object and add it to the responee
		my $record = SRU::Response::Record->new(recordSchema => 'info:srw/schema/1/dc-v1.1', recordData => $results[$i]);
		$response->addRecord($record);
	
	}
	
}

# done; output the result
print $cgi->header('text/xml');
print $response->asXML();
exit;


######################################################
# local subroutines; should I make these into methods?

sub record {

	my $email = 'emorgan@nd.edu';
	
	return <<EOF
<explain xmlns="http://explain.z3950.org/dtd/2.0/">
	<serverInfo protocol='SRU' version='1.1'>
		<host>
			alert.ockham.org
		</host>
		<port>
			80
		</port>
		<database>
			sru-server.cgi
		</database>
	</serverInfo>
	<databaseInfo>
		<title lang='en' primary='true'>
			OCKHAM Alerting Service
		</title>
		<description lang='en' primary='true'>
			This is an (incompletely written) implementation of an SRU interface providing access to an OCKHAM Alerting service. The alerting service works by: 1) harvesting content from OAI providers, 2) saving the content to a centralized store, 3) indexing the content, and 4) providing access to the index. For more information about OCKHAM and the proposed alerting service see http://www.ockham.org/.
		</description>
		<author lang='en' primary='true'>
			Eric Lease Morgan, University Libraries of Notre Dame
		</author>
		<contact lang='en' primary='true'>
			Eric Lease Morgan ($email)
		</contact>
		<extent lang='en' primary='true'>
			At the present time, this database is not very complete; this database is an example implementation right now.
		</extent>
		<history lang='en' primary='true'>
			This implementation was begun September 5, 2004, and the last modification was on December 23, 2004. It's implementation is a part of a United States National Science Foundation Digital Library project called Ockham. See: http://www.ockham.org/.
		</history>
		<langUsage lang='en' primary='true' codes='en'>
			All the records in this database are in English.
		</langUsage>
		<implementation lang='en' primary='true'>
			This SRU server is written in Perl through the use of three API's. One for SRU (http://http://search.cpan.org/dist/SRU/). One for CQL (http://search.cpan.org/dist/CQL-Parser/), and one for the underlying indexer/search engine, swish-e (http://www.swish-e.org/).
		</implementation>
		<links>
			<link type='www'>
			http://alert.ockham.org/
			</link>
			<link type='sru'>
			http://alert.ockham.org/sru-server.cgi
			</link>
			<link type='icon'>
			http://alert.ockham.org/favicon.ico
			</link>
		</links>
	</databaseInfo>
	<metaInfo>
		<dateModified>
			2004-12-23 16:12:04
		</dateModified>
	</metaInfo>
	<indexInfo>
		<set identifier='info:srw/cql-context-set/1/dc-v1.1' name='dc' />
		<index>
			<title>
				title
			</title>
			<map>
				<name set='dc'>
					title
				</name>
			</map>
		</index>
		<index>
			<title>
				creator
			</title>
			<map>
				<name set='dc'>
					creator
				</name>
			</map>
		</index>
		<index>
			<title>
				description
			</title>
			<map>
				<name set='dc'>
					description
				</name>
			</map>
		</index>
	</indexInfo>
	<schemaInfo>
		<schema identifier='info:srw/schema/1/dc-v1.1' sort='false' name='dc'>
			<title>
				Dublin Core
			</title>
		</schema>
	</schemaInfo>
	<configInfo>
		<default type='numberOfRecords'>
			250
		</default>
	</configInfo>
</explain>

EOF

}


sub search {

	# initialize
	my $query = $_[0];
	my @results;
	
	# open the index
	my $swish = SWISH::API->new(SWISH_INDEX);
	
	# do a search with the command line input
	my $results = $swish->Query($query);
	
	# check for hits
	if ($results->Hits > 0) {
	
		# process each result
		while (my $r = $results->NextResult) {
		
			# get the id
			my $resource = Ockham::Alert::Resource->retrieve($r->Property("swishdocpath"));
			
			# push on to results
			if ($resource) {push @results, '<srw_dc:dc xmlns="http://www.w3.org/TR/xhtml1/strict" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:srw_dc="info:srw/schema/1/dc-v1.1"><dc:title>' . &escape_entities($resource->field('title')) . '</dc:title><dc:identifier>' . &escape_entities($resource->field('identifier')) . '</dc:identifier><dc:description>' . &escape_entities($resource->field('description')) . '</dc:description><dc:creator>' . &escape_entities($resource->field('creator')) . '</dc:creator><dc:provider>' . &escape_entities($resource->provider->name) . '</dc:provider><dc:date>' . &escape_entities($resource->field('date')) . '</dc:date></srw_dc:dc>'; }
			
		}
				
	}
	
	# done; return it
	return @results;
	
}


sub escape_entities {

	# get the input
	my $s = shift;
	
	# escape
	$s =~ s/&/&amp;/g;
	$s =~ s/</&lt;/g;
	$s =~ s/>/&gt;/g;
	$s =~ s/"/&quot;/g;
	$s =~ s/'/&apos;/g;

	# done
	return $s;
	
}


sub error {

	print $cgi->header();
	print $SRU::Error;
	exit;
	
}

